'use strict';

var helperPluginUtils = require('@babel/helper-plugin-utils');
var t = require('@babel/types');

const {
  AvailableD3Modules
} = require('./resources');

function isTypeImport(node) {
  return node.importKind && node.importKind !== 'value';
}
function isAvailableSource(node) {
  return t.isStringLiteral(node.source) && AvailableD3Modules.indexOf(node.source.value) >= 0;
}
function isParsableImportSpecifiers(node) {
  return node.specifiers.every(spec => {
    const isParsable = !isTypeImport(spec) && !t.isImportDefaultSpecifier(spec);
    return isParsable;
  });
}

const {
  D3Modules
} = require('./resources');

var index = helperPluginUtils.declare(api => {
  api.assertVersion(7);
  return {
    name: 'transform-d3-imports',
    visitor: {
      ImportDeclaration(path) {
        const {
          node
        } = path;
        const escape = isTypeImport(node) || !isAvailableSource(node) || !isParsableImportSpecifiers(node);

        if (escape) {
          return;
        }

        const d3Module = D3Modules[node.source.value];
        const parsedImportSourceMap = {};
        let containsUnknownImport = false;
        node.specifiers.forEach(spec => {
          const newImportSource = d3Module[spec.imported.name];

          if (!newImportSource) {
            containsUnknownImport = true;
          } else if (typeof newImportSource === 'string') {
            parsedImportSourceMap[newImportSource] = [spec.local.name];
          } else {
            const [source, opts] = newImportSource;
            parsedImportSourceMap[source] = parsedImportSourceMap[source] || [];
            parsedImportSourceMap[source].push([spec.local.name, opts.as]);
          }
        });

        if (containsUnknownImport) {
          return;
        }

        const replacementImportNodes = Object.entries(parsedImportSourceMap).map(([source, imports]) => {
          const specifiers = imports.map(pair => typeof pair === 'string' ? t.importDefaultSpecifier(t.identifier(pair)) : t.importSpecifier(t.identifier(pair[0]), t.identifier(pair[1])));
          return t.importDeclaration(specifiers, t.stringLiteral(source));
        });
        path.replaceWithMultiple(replacementImportNodes);
      }

    }
  };
});

module.exports = index;
//# sourceMappingURL=index.js.map
